#define STATES 7
#define HUMIDITY_SENSOR_DIGITAL_PIN 4
#define DEBUG

#ifdef DEBUG
#define DEBUG_SERIAL(x) Serial.begin(x)
#define DEBUG_PRINT(x) Serial.print(x)
#define DEBUG_PRINTLN(x) Serial.println(x)
#else
#define DEBUG_SERIAL(x)
#define DEBUG_PRINT(x) 
#define DEBUG_PRINTLN(x) 
#endif

#include <avr/pgmspace.h>
#include <Wire.h>
#include <Time.h>
#include <SPI.h>
#include <MySensor.h>
#include <LiquidCrystal_I2C.h>
#include <DS3232RTC.h> 
//
#define RADIO_ID 11
#define CHILD_ID_SCENE 3
#define CHILD_ID_LED 4
//
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);  // set the LCD address to 0x27 for a 16 chars and 2 line display
//
void (*lcdDisplay[STATES])();
//
byte state = 0;
byte lastState;
byte timeCounter = 0;
unsigned long lastTime;
unsigned long refreshInterval = 3000UL;
unsigned long lastClockSet;
boolean isMessage = false;
float insideTemperature;
float humidity;
int OutdoorTemp = -99;
int OutdoorHumidity = -99;
int todayHigh = -99; 
int todayLow = -99;
String conditions = "Not yet Reported";
String tomorrowWeather = "Not yet Reported";
String messageOfTheDay = "**Hello There!**";
int HotTubTemp = -99;
unsigned long motdTimer;
int ledStatus;// to toggle LCD backlight led
boolean buttonPushed = false;
//
MySensor gw;
//
MyMessage msgOn(CHILD_ID_SCENE, V_SCENE_ON);
MyMessage msgOff(CHILD_ID_SCENE, V_SCENE_OFF);
MyMessage msgVAR1(CHILD_ID_SCENE, V_VAR1);// outdoor temperature
MyMessage msgVAR2(CHILD_ID_SCENE, V_VAR2);// outdoor humidity
MyMessage msgVAR3(CHILD_ID_SCENE, V_VAR3);//today's low
MyMessage msgVAR4(CHILD_ID_SCENE, V_VAR4);// today's high
MyMessage msgVAR5(CHILD_ID_SCENE, V_VAR5);//conditions
//
MyMessage lightMsg(CHILD_ID_LED, V_LIGHT);
MyMessage msgMOTD(CHILD_ID_LED, V_VAR1); // message of the day
MyMessage msgAlarm(CHILD_ID_LED, V_VAR2); // alarm status
MyMessage msgBrite(CHILD_ID_LED, V_VAR3); // led briteness (nice idea!!!!)
MyMessage msgHTTemp(CHILD_ID_LED, V_VAR4); // Hot Tub Temp
//
void setup()  
{ 
  DEBUG_SERIAL(115200);
  DEBUG_PRINTLN(F("Serial started"));
  attachInterrupt(1, PushButton, CHANGE);
  //
  lcdDisplay[0] = lcdDisplay0;
  lcdDisplay[1] = lcdDisplay1;
  lcdDisplay[2] = lcdDisplay2;
  lcdDisplay[3] = lcdDisplay3;
  lcdDisplay[4] = lcdDisplay4;
  lcdDisplay[5] = lcdDisplay5;
  lcdDisplay[6] = lcdDisplay6;
  //
  gw.begin(getVariables, RADIO_ID);
  gw.sendSketchInfo("WeatherClock", "1.0");
  setSyncProvider (RTC.get);
  gw.present(CHILD_ID_SCENE, S_SCENE_CONTROLLER);
  gw.present( CHILD_ID_LED, S_LIGHT);
  //
  lcd.begin(20,4);
  lcd.clear();
  lcd.backlight();
  lcd.setCursor(0, 0);
  lcd.print("Syncing Time");
  lcd.setCursor(0, 1);
  int clockCounter = 0;
  while(timeStatus() == timeNotSet && clockCounter < 60)
  {
    gw.process();
    gw.requestTime(receiveTime);
    Serial.println("getting Time");
    delay(1000);
    lcd.print(".");
    clockCounter++;
    if (clockCounter > 16)
    {
      lcd.clear();
      lcd.print(F("**Failed Clock**"));
      lcd.setCursor(0,1);
      lcd.print(F("*Syncronization*"));
      delay(2000);
      break;
    }
  }
  lcd.clear();
  lastTime = millis();
}
void loop()      
{
  gw.process();
  if (millis() - lastClockSet >= 60000UL)
  {
    gw.requestTime(receiveTime);
    lastClockSet = millis();
  }
  if (millis() - lastTime >= refreshInterval)
  {
    state++;
    if (state > STATES - 1) state = 0;
    DEBUG_PRINTLN(F("State:")); 
    DEBUG_PRINTLN(state);
    lastTime += refreshInterval;
    getTempHumidity();
  }
  if (state != lastState) 
  {
    fastClear();
    lcdDisplay[state]();
  }
  lastState = state;
  if (buttonPushed)
  {
    activateScene();
  }
  if (isMessage)
  {
    if (millis() - motdTimer > 86400 * 1000UL)
    {
      isMessage = false;
    }
  }
}
void fastClear()
{
  lcd.setCursor(0,0);
  lcd.clear();
  lcd.setCursor(0,1);
  lcd.clear();
  lcd.setCursor(0,2);
  lcd.clear();
  lcd.setCursor(0,3);
  lcd.clear();
}
//
void lcdDisplay0()
{
  lcd.setCursor(0,0);
  lcd.print(F("Time: "));
  if (hourFormat12() < 10) lcd.print("0");
  lcd.print(hourFormat12());
  lcd.print(":");
  if (minute() < 10) lcd.print("0");
  lcd.print(minute());
  if (isAM()) lcd.print(F("am"));
  else lcd.print(F("pm"));
  DEBUG_PRINT(F("Time:"));
  DEBUG_PRINTLN(hourFormat12());
  lcd.setCursor(0,1);
  lcd.print(F("Date: "));
  if (month() < 10) lcd.print("0");
  lcd.print(month());
  lcd.print("/");
  if (day() < 10) lcd.print("0");
  lcd.print(day());
  lcd.print("/");
  lcd.print(year());
  DEBUG_PRINTLN(F("Date: ")); 
  DEBUG_PRINT(month()); 
  DEBUG_PRINT("/"); 
  DEBUG_PRINT(day()); 
  DEBUG_PRINT("/"); 
  DEBUG_PRINTLN(year());
}
void lcdDisplay1()
{
  lcd.setCursor(0,0);
  lcd.print(F(" Indoor Temp:"));
  lcd.print(int(round(insideTemperature)));
  lcd.print(char(223));
  DEBUG_PRINT(F("Indoor Temp:")); 
  DEBUG_PRINT(int(round(insideTemperature))); 
  DEBUG_PRINTLN(F("F"));
  lcd.setCursor(0,1);
  lcd.print(F("Hot Tub:"));
  lcd.print(HotTubTemp);
  lcd.print(char(223));
  DEBUG_PRINT(F("Hot Tub:"));
  DEBUG_PRINT(HotTubTemp);
  DEBUG_PRINT(F("F"));
}
void lcdDisplay2()
{
  lcd.setCursor(0,0);
  lcd.print("Outdoor Temp:"); 
  lcd.print(OutdoorTemp); 
  lcd.print(char(223));
  DEBUG_PRINT(F("Outdoor Temp:"));
  DEBUG_PRINTLN(OutdoorTemp);
  lcd.setCursor(0,1);
  lcd.print(F("    Humidity:")); 
  lcd.print(OutdoorHumidity); 
  lcd.print(F("%"));
  DEBUG_PRINT(F("    Humidity:"));
  DEBUG_PRINTLN(OutdoorHumidity);
}
void lcdDisplay3()
{
  lcd.setCursor(0,0);
  lcd.print(F("Today's High:"));
  lcd.print(todayHigh); 
  lcd.print(char(223));
  DEBUG_PRINT(F("Today's High: "));
  DEBUG_PRINTLN(todayHigh);
  lcd.setCursor(0,1);
  lcd.print(F("         Low:"));
  lcd.print(todayLow); 
  lcd.print(char(223));
  DEBUG_PRINT(F("Today's Low: "));
  DEBUG_PRINTLN(todayLow);
}
void lcdDisplay4()
{
  lcd.setCursor(0,0);
  lcd.print(F("Today's Weather"));
  DEBUG_PRINTLN(F("Today's Weather: "));
  lcd.setCursor(0,1);
  lcd.print(conditions);
  DEBUG_PRINTLN(conditions);
}
void lcdDisplay5()
{
  lcd.setCursor(0,0);
  lcd.print(F("Forcast Tomorrow"));
  DEBUG_PRINTLN(F("Tomorrow's Forecast: "));
  lcd.setCursor(0,1);
  lcd.print(tomorrowWeather);
  DEBUG_PRINTLN(tomorrowWeather);
}
void lcdDisplay6()
{
  if (isMessage)
  {
    lcd.setCursor(0,0);
    lcd.print(F("**NEW  MESSAGE**"));
    DEBUG_PRINTLN(F("****Message****"));
    lcd.setCursor(0,1);
    lcd.print(messageOfTheDay);
    DEBUG_PRINTLN(messageOfTheDay);
    motdTimer = millis();
  }
  else
  {
    lcd.setCursor(0,0);
    lcd.print(F("****Welcome!****"));
    DEBUG_PRINTLN(F("****Message****"));
    lcd.setCursor(0,1);
    lcd.print(F("Have a Nice Day!"));
    DEBUG_PRINTLN(F("Have a Nice Day"));
  }
}
//
void getTempHumidity()
{
  insideTemperature = ((RTC.temperature()/4));
  if (isnan(insideTemperature)) 
  {
    DEBUG_PRINTLN(F("Failed reading temperature from RTC"));
  } 
}
//
void receiveTime(unsigned long controllerTime)
{
  // Ok, set imcoming time
  DEBUG_PRINTLN(F("Time value received: "));
  DEBUG_PRINTLN(controllerTime);
  RTC.set(controllerTime);
  controllerTime = true;
}
//
void PushButton()
{
  static unsigned long last_interrupt_time = 0;
  unsigned long interrupt_time = millis();
  if (interrupt_time - last_interrupt_time > 200)
  {
    buttonPushed = true;
  }
  last_interrupt_time = interrupt_time;
}
//
void activateScene()
{
  DEBUG_PRINTLN(F("ButtonPushed"));
  fastClear();
  for (byte i = 0; i < 10; i++)
  {
    lcd.noBacklight();
    delay(50);
    lcd.backlight();
    delay(50);
  }
  lcd.setCursor(0,0);
  lcd.print(F(" A/C Boost Mode "));
  lcd.setCursor(0,1);
  lcd.print(F("**** ACTIVE ****"));
  delay(2000);
  buttonPushed = false;
  lastTime = millis(); //Reset the timer to even out display interval
  ledStatus = gw.loadState(CHILD_ID_LED);
  if (ledStatus > 0)
  {
    lcd.backlight();
  }
  else
  {
    lcd.noBacklight();
  }
}
//
void getVariables(const MyMessage &message)
{
  if (message.sensor == CHILD_ID_SCENE)
  { 
    if (message.type == V_VAR1)
    {
      OutdoorTemp = atoi(message.data);
      DEBUG_PRINTLN(F("OutdoorTemp recieved:"));
      DEBUG_PRINTLN(OutdoorTemp);
    }
    if (message.type == V_VAR2)
    {
      OutdoorHumidity = atoi(message.data);
      DEBUG_PRINT(F("OutdoorHumidity recieved:"));
      DEBUG_PRINTLN(OutdoorHumidity);
    }
    if (message.type == V_VAR3)
    {
      todayLow = atoi(message.data);
      DEBUG_PRINT(F("Received Today's LOW:"));
      DEBUG_PRINTLN(todayLow);
    }

    if (message.type == V_VAR4)
    {
      todayHigh = atoi(message.data);
      DEBUG_PRINT(F("Received Today's HIGH:"));
      DEBUG_PRINTLN(todayHigh);
    }
    if (message.type == V_VAR5)
    {
      String newMessage = String(message.data);
      int locator = newMessage.indexOf("@");
      newMessage = newMessage.substring(0, locator);
      conditions = newMessage;
      DEBUG_PRINT(F("Received today's Conditions:"));
      DEBUG_PRINTLN(conditions);
    }
  }
  if (message.sensor == CHILD_ID_LED)
  {
    if (message.type == V_LIGHT)
    {
      int ledState = atoi(message.data);
      if (ledState > 0)
      {
        lcd.backlight();
      }
      else
      {
        lcd.noBacklight();
      }
    }
    if (message.type == V_VAR1)
    {
      // led briteness (nice idea!!!!)
    }
    if (message.type == V_VAR2)
    {
      // Extended Forecast
      String newMessage = String(message.data);
      int locator = newMessage.indexOf("@");
      newMessage = newMessage.substring(0, locator);
      tomorrowWeather = newMessage;
      DEBUG_PRINT(F("Received Two Day Forecast:"));
      DEBUG_PRINTLN(tomorrowWeather);
    }
    if (message.type == V_VAR3)
    {
      // message of the day
      String newMessage = String(message.data);
      int locator = newMessage.indexOf("@");
      newMessage = newMessage.substring(0, locator);
      messageOfTheDay = newMessage;
      DEBUG_PRINT(F("Received Message of the Day:"));
      DEBUG_PRINTLN(messageOfTheDay);
      isMessage = true;
    }
    if (message.type == V_VAR4)
    {
      HotTubTemp = atoi(message.data);
      DEBUG_PRINT(F("HotTubTemp recieved:"));
      DEBUG_PRINTLN(HotTubTemp);
    }
  }
}
